from django.shortcuts import render
from django.views.generic import View
from django.conf import settings
from django.db import close_old_connections
from django_redis import get_redis_connection
from apps.OSVersionCompatibilityAnalysis.models import Ostransfer
from apps.account.utils import has_host_perm
from apps.host.models import Host
from apps.setting.utils import AppSetting
from libs import json_response, JsonParser, Argument, auth
from libs.utils import str_decode, human_seconds_time
from concurrent import futures
from threading import Thread
import subprocess
import tempfile
import uuid
import json
import time
import os
import glob

# Create your views here.

class OSCompatibilityCheck(View):
    
    @auth('os.check.view')
    def get(self, request):
        records = Ostransfer.objects.filter(user=request.user)
        return json_response([x.to_view() for x in records])
    
    @auth('os.check.view')
    def post(self, request):
        data = request.POST.get('data')
        form, error = JsonParser(
            Argument('host_ids', type=list, filter=lambda x: len(x), help='请选择目标主机'),
            Argument('host_hostnames', type=list, help='请选择目标主机'),
        ).parse(data)
        if error is None:
            if not has_host_perm(request.user, form.host_ids):
                return json_response(error='无权访问主机，请联系管理员')
            host_id = None
            token = uuid.uuid4().hex
            Ostransfer.objects.create(
                user=request.user,
                digest=token,
                host_id=host_id,
                host_ids=json.dumps(form.host_ids),
                host_hostnames=json.dumps(form.host_hostnames),
            )
            return json_response(token)
        return json_response(error=error)


    @auth('os.check.view')
    def patch(self, request):
        form, error = JsonParser(
            Argument('token', help='参数错误')
        ).parse(request.body)
        if error is None:
            task = Ostransfer.objects.get(digest=form.token)
            Thread(target=_dispatch_sync, args=(task,)).start()
        return json_response(error=error)

    
def _dispatch_sync(task):
    rds = get_redis_connection()
    threads = []
    max_workers = max(10, os.cpu_count() * 5)
    with futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
        for host in Host.objects.filter(id__in=json.loads(task.host_ids)):
            t = executor.submit(_do_sync, rds, task, host)
            t.token = task.digest
            t.key = host.id
            threads.append(t)
        for t in futures.as_completed(threads):
            exc = t.exception()
            if exc:
                rds.publish(
                    t.token,
                    json.dumps({'key': t.key, 'status': -1, 'data': f'\x1b[31mException: {exc}\x1b[0m'})
                )
    close_old_connections()


def _do_sync(rds, task, host):
    token = task.digest
    rds.publish(token, json.dumps({'key': host.id, 'data': '\r\n\x1b[36m### Executing ...\x1b[0m\r\n'}))

    command_str = f'run-compatibility-analysis-tool.sh'
    print(command_str)
    task = Ostransfer.objects.get(digest=token)
    data = dict(
        key=host.id,
        name=host.name,
        token=token,
        interpreter="sh",
        hostname=host.hostname,
        port=host.port,
        username=host.username,
        command=command_str,
        pkey=host.private_key,
        params="",
        term={'width': 100, 'height': 100}
    )
    rds.rpush(settings.EXEC_WORKER_KEY, json.dumps(data))
