
import os
from apps.setting.utils import AppSetting
from libs import json_response, JsonParser, Argument

DEFAULT_CONFIG_DIR = '/opt/bitnami/prometheus/conf/static_conf/'

def prometheus_config(request):
    form, error = JsonParser(
        Argument('target', help='请输入监控目标'),
        Argument('lables', type=dict, required=False),
    ).parse(request.body)

    if error is None:
        try:
            prometheus_exporter_config_dir=AppSetting.get_default('prometheus_exporter_config_dir', DEFAULT_CONFIG_DIR)

            # 检查目录是否存在，如果不存在则创建
            if not os.path.exists(prometheus_exporter_config_dir):
                os.makedirs(prometheus_exporter_config_dir)

            # 写入配置文件
            with open(f'{prometheus_exporter_config_dir}/{form.target}.yml', 'w') as f:
                f.write(f'- targets: [{form.target}]')
                if form.lables:
                    f.write('\n  labels:')
                    for k, v in form.lables.items():
                        f.write(f'\n    {k}: {v}')

        except IOError:
            return json_response({'success': False, 'msg': 'write file error'})
        except Exception as e:
            return json_response({'success': False, 'msg': f'unexpected error：{str(e)}'})
            
    return json_response({'success': True, 'msg': 'config generate success'})
