from django.db import models
from apps.account.models import User

from libs import ModelMixin, human_datetime
import json

class Cve(models.Model, ModelMixin):
    """cve信息"""
    cve = models.CharField(max_length=255)
    """cve编码"""
    sa_id = models.CharField(max_length=255)
    """sa编码"""
    sa_link = models.CharField(max_length=255)
    """sa链接"""
    sa_title = models.CharField(max_length=255)
    """sa标题"""
    sa_summary = models.TextField()
    """sa摘要"""
    sa_cvss = models.DecimalField(max_digits=3, decimal_places=1, null=True)
    """sa cvss评分"""
    sa_release_time = models.CharField(max_length=20)
    """sa发布时间"""
    updated_at = models.CharField(max_length=20, default=human_datetime)
    """更新时间"""
    created_at = models.CharField(max_length=20, default=human_datetime)
    """创建时间"""

    def __repr__(self) -> str:
        return super().__repr__()
    
    class Meta:
        db_table = 'cves'
        ordering = ('-id',)


class Patch(models.Model, ModelMixin):
    """补丁信息"""
    cve = models.CharField(max_length=255)
    """cve"""
    platform = models.CharField(max_length=255)
    """平台"""
    arch = models.CharField(max_length=255)
    """架构"""
    package_name = models.CharField(max_length=255)
    """包名"""
    package_file_name = models.CharField(max_length=255)
    """包文件名"""
    version = models.CharField(max_length=255)
    """版本"""
    cvss = models.DecimalField(max_digits=3, decimal_places=1, null=True)
    """sa cvss评分"""
    updated_at = models.CharField(max_length=20, default=human_datetime)
    """更新时间"""
    created_at = models.CharField(max_length=20, default=human_datetime)
    """创建时间"""

    def __repr__(self) -> str:
        return super().__repr__()
    
    class Meta:
        db_table = 'patches'
        ordering = ('-id',)


class HostScan(models.Model, ModelMixin):
    """
    主机扫描记录
    """
    host_id = models.IntegerField()
    """主机id"""
    is_fixed = models.BooleanField(default=False, null=True)
    """是否修复"""
    patch_messages = models.TextField(null=True)
    """修复消息"""
    updated_at = models.CharField(max_length=20, default=human_datetime)
    """更新时间"""
    created_at = models.CharField(max_length=20, default=human_datetime)
    """创建时间"""

    def __repr__(self) -> str:
        return super().__repr__()
    
    class Meta:
        db_table = 'host_scans'
        ordering = ('-id',)

class HostScanPatch(models.Model, ModelMixin):
    """主机扫描id"""
    package_name = models.CharField(max_length=255)
    """包名"""
    current_version = models.CharField(max_length=255)
    """当前版本"""
    target_version = models.CharField(max_length=255)
    """目标版本"""
    cve = models.CharField(max_length=255, null=True)
    """cve"""
    cvss = models.DecimalField(max_digits=3, decimal_places=1, null=True)
    """cvss评分"""
    updated_at = models.CharField(max_length=20, default=human_datetime)
    """更新时间"""
    created_at = models.CharField(max_length=20, default=human_datetime)
    """创建时间"""
    hs = models.ForeignKey(HostScan, models.CASCADE, related_name='host_scan_patches')
    """主机扫描"""
    is_fixed = models.BooleanField(default=False, null=True)

    def __repr__(self) -> str:
        return super().__repr__()
    
    class Meta:
        db_table = 'host_scan_patches'
        ordering = ('-id',)

class HostScanCve(models.Model, ModelMixin):
    """主机扫描id"""
    host_id = models.IntegerField()
    """主机id"""
    cve = models.CharField(max_length=255)
    """cve"""
    cvss = models.DecimalField(max_digits=3, decimal_places=1, null=True)
    """cvss评分"""
    updated_at = models.CharField(max_length=20, default=human_datetime)
    """更新时间"""
    created_at = models.CharField(max_length=20, default=human_datetime)
    """创建时间"""
    hs = models.ForeignKey(HostScan, models.CASCADE, related_name='host_scan_cves')
    """主机扫描"""
    is_fixed = models.BooleanField(default=False, null=True)

    def __repr__(self) -> str:
        return super().__repr__()
    
    class Meta:
        db_table = 'host_scan_cves'
        ordering = ('-id',)




class Manualtransfer(models.Model, ModelMixin):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    digest = models.CharField(max_length=32, db_index=True)
    host_id = models.IntegerField(null=True)
    src_dir = models.CharField(max_length=255)
    dst_dir = models.CharField(max_length=255)
    host_ids = models.TextField()
    host_hostnames = models.TextField()
    updated_at = models.CharField(max_length=20, default=human_datetime)

    def to_view(self):
        tmp = self.to_dict()
        tmp['host_ids'] = json.loads(self.host_ids)
        tmp['host_hostnames'] = json.loads(self.host_hostnames)
        return tmp

    class Meta:
        db_table = 'manual_transfer'
        ordering = ('-id',)
