from django.core.management.base import BaseCommand
import socketserver
from apps.syslog.models import Syslog
import re

import json

class SyslogUDPHandler(socketserver.BaseRequestHandler):
    def handle(self):
        # data = bytes.decode(self.request[0].strip())
        # print("data", data);
        # return None
        try:
            data = json.loads(bytes.decode(self.request[0].strip()))
            socket = self.request[1]
            host, _ = self.client_address
            Syslog.objects.create(
                host=host,
                pri = data['pri'],
                syslogfacility = data['syslogfacility'],
                syslogseverity = data['syslogseverity'],
                timegenerated = data['timegenerated'],
                timereported = data['timereported'],
                hostname = data['hostname'],
                programname = data['programname'],
                procid = data['procid'],
                msg = data['msg'],
                app_name = data['app-name'],
                msgid = data['msgid'],
                protocol_version = data['protocol-version'],
                structured_data = data['structured-data'],
                fromhost = data['fromhost'],
                fromhost_ip = data['fromhost-ip'],
                myhostname = data['myhostname'],
                now = data['now'],
                year = data['year'],
                month = data['month'],
                day = data['day'],
                hour = data['hour'],
                minute = data['minute'],
            )
        except json.JSONDecodeError as e:
            print(f"JSON decode error: {e}")
            print("data", bytes.decode(self.request[0].strip()));
    
        # print(data)
        #data = self.parse_syslog_message(data)

        # data1 = json.loads(bytes.decode(self.request[0].strip()))

        # print("data1", data1);
        # print("data1['msg']", data1['msg']);
       

    def parse_syslog_message(self, message):
        # 根据你的syslog格式进行解析
        # 这里只是一个示例，具体解析方式取决于你的syslog格式
        print(message)
        msg_time, hostname, tag, priority, msg = message.split('_$_$_', 5)
        print("------------------------------")
        print("msg_time", msg_time)
        print("hostname", hostname)
        print("tag", tag)
        print("priority", priority)
        app, pid = self.parse_app_pid(tag)

        print("app", app)
        print("pid", pid)
        print("msg", msg)
        print("------------------------------")
        return {
            'msg_time': msg_time,
            'app': app,
            'pid': pid,
            'priority': priority,
            'msg': msg
        }

    def parse_app_pid(self, s):
        match = re.match(r'(.+)\[(\d+)\]', s)
        if match:
            app = match.group(1)
            pid = match.group(2)
            return app, pid
        else:
            return None

class Command(BaseCommand):
    help = 'Starts a UDP server to receive syslog messages'

    def handle(self, *args, **options):
        HOST, PORT = '0.0.0.0', 514
        server = socketserver.UDPServer((HOST, PORT), SyslogUDPHandler)

        # Run the server
        try:
            server.serve_forever(poll_interval=0.5)
        except KeyboardInterrupt:
            print("Server stopped.")
