from django.http import HttpResponse
from .models import Syslog
from django.views import View
from django.db.models import Q
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger


from libs import json_response, JsonParser, Argument, auth


ITEMS_PER_PAGE = 10

def syslog_receiver(request):
    if request.method == 'POST':
        # 假设syslog消息以特定的格式发送
        host = request.META.get('REMOTE_ADDR')
        message = request.body.decode('utf-8')

        # 保存消息到数据库
        Syslog.objects.create(host=host, message=message)

        return HttpResponse('Syslog message received', status=200)
    else:
        return HttpResponse('Invalid request', status=400)

class SyslogView(View):

    # def get(self, request):
    #     form, error = JsonParser(
    #         Argument('id', type=int, required=False)
    #     ).parse(request.GET)
    #     if error is None:
    #         if request.user.is_supper:
    #             syslogs = Syslog.objects.all()
    #         else:
    #             ids = request.user.deploy_perms['apps']
    #             syslogs = Syslog.objects.filter(id__in=ids)

    #         if form.id:
    #             syslogs = Syslog.filter(pk=form.id).first()
    #             return json_response(syslogs)
            
    #         return json_response(syslogs)
    #     return json_response(error=error)

    @auth('syslog.syslog.view')
    def get(self, request):
        # 获取查询参数
        host = request.GET.get('host')
        start_time = request.GET.get('start_time')
        end_time = request.GET.get('end_time')
        page = int(request.GET.get('page', 1))  # 获取当前页数，默认为第1页
        size = int(request.GET.get('size', ITEMS_PER_PAGE))

        # 构建查询条件
        conditions = []
        if host:
            conditions.append(Q(host__contains=host))
        if start_time and end_time:
            conditions.append(Q(updated_at__gte=start_time))
            conditions.append(Q(updated_at__lte=end_time))
        if start_time and not end_time:
            conditions.append(Q(updated_at__gte=start_time))
        if not start_time and end_time:
            conditions.append(Q(updated_at__lte=end_time))

        # 执行查询，按time倒序，ip正序排序
        event_list = Syslog.objects.filter(*conditions).order_by('-updated_at', 'host')

        # 创建一个 Paginator 对象，提供预先计算好的计数值
        paginator = Paginator(event_list, size) 
        total = paginator.count

        # 获取该页的对象，执行数据库查询
        syslogs = paginator.get_page(page)

        # 序列化查询结果
        syslogs = ([{"created_at": syslog.created_at, 
                    "updated_at": syslog.updated_at, 
                    "msg_time": syslog.msg_time,
                    "host": syslog.host,
                    "pri": syslog.pri,
                    "syslogfacility": syslog.syslogfacility,
                    "syslogseverity": syslog.syslogseverity,
                    "timegenerated": syslog.timegenerated,
                    "timereported": syslog.timereported,
                    "hostname": syslog.hostname,
                    "programname": syslog.programname,
                    "procid": syslog.procid,
                    "msg": syslog.msg,
                    "app_name": syslog.app_name,
                    "msgid": syslog.msgid,
                    "protocol_version": syslog.protocol_version,
                    "structured_data": syslog.structured_data,
                    "fromhost": syslog.fromhost,
                    "fromhost_ip": syslog.fromhost_ip,
                    "myhostname": syslog.myhostname,
                    "now": syslog.now,
                    "year": syslog.year,
                    "month": syslog.month,
                    "day": syslog.day,
                    "hour": syslog.hour,
                    "minute": syslog.minute} 
                   for syslog in syslogs])

        # 计算总页数（假设数据库中总条数已知）
        total_pages = (total + size - 1) // size

        # 返回结果，包括分页信息
        response_data = {
            "syslogs": syslogs,
            "total": total,
            "total_pages": total_pages,
            "current_page": page,
        }

        return json_response(response_data)
