import os
import re

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'vk0do47)egwzz!uk49%(y3s(fpx4+ha@ugt-hcv&%&d@hwr&p7'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']

# Application definition

INSTALLED_APPS = [
    'django_crontab',
    'apps.account',
    'apps.host',
    'apps.setting',
    'apps.exec',
    'apps.schedule',
    'apps.monitor',
    'apps.alarm',
    'apps.config',
    'apps.app',
    'apps.deploy',
    'apps.notify',
    'apps.repository',
    'apps.home',
    'apps.integration',
    'channels',
    'apps.abrtMonitor',
    'apps.ebpf',
    'apps.patch',
    'apps.syslog',
    'apps.HardwareCompatibilityCheck',
    'apps.softwarecheck',
    'apps.OSVersionCompatibilityAnalysis'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.middleware.common.CommonMiddleware',
    'libs.middleware.AuthenticationMiddleware',
    'libs.middleware.HandleExceptionMiddleware',
]

ROOT_URLCONF = 'nfs_ops.urls'

WSGI_APPLICATION = 'spnfs_opsug.wsgi.application'
ASGI_APPLICATION = 'nfs_ops.routing.application'

# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ATOMIC_REQUESTS': True,
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/1",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    }
}

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels_redis.core.RedisChannelLayer",
        "CONFIG": {
            "hosts": [("127.0.0.1", 6379)],
            "capacity": 1000,
            "expiry": 120,
        },
    },
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': False,
    },
]

TOKEN_TTL = 8 * 3600
SCHEDULE_KEY = 'nfs_ops:schedule'
SCHEDULE_WORKER_KEY = 'nfs_ops:schedule:worker'
MONITOR_KEY = 'nfs_ops:monitor'
MONITOR_WORKER_KEY = 'nfs_ops:monitor:worker'
EXEC_WORKER_KEY = 'nfs_ops:exec:worker'
REQUEST_KEY = 'nfs_ops:request'
BUILD_KEY = 'nfs_ops:build'
REPOS_DIR = os.path.join(os.path.dirname(os.path.dirname(BASE_DIR)), 'repos')
BUILD_DIR = os.path.join(REPOS_DIR, 'build')
TRANSFER_DIR = os.path.join(BASE_DIR, 'storage', 'transfer')

# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'Asia/Shanghai'

USE_I18N = True

USE_L10N = True

USE_TZ = False

AUTHENTICATION_EXCLUDES = (
    '/account/login/',
    '/setting/basic/',
    re.compile('/integration/.*'),
    re.compile('/abrtMonitor/.*'),
    re.compile('/ebpf/.*'),
    # re.compile('/syslog/.*'),
    re.compile('/apis/.*'),
    re.compile('/host/vnc/get_vnc_host_port/.*'),
)

BIND_IP_EXCLUDES = (
    re.compile('/host/.*'),
)

CRONJOBS = [
    ('*/10 * * * *', 'apps.ebpf.cron.scheduled_task', '>>/tmp/nfs-ops-eventlog-scheduler.log'),
]

SPUG_VERSION = 'v3.3.2'

# override default config
try:
    from nfs_ops.overrides import *
except ImportError:
    pass
