#!/bin/bash

set -e

HOSTNAME=$1
NFS_OPS_BASE_URL=$2
ARCH=$(uname -m) 

# 判读 ARCH，如果 不是 x86_64 或 aarch64，则退出脚本
if [ "$ARCH" != "x86_64" ] && [ "$ARCH" != "aarch64" ]; then
    echo "不支持的架构：$ARCH"
    exit 1
fi

# 根据 $ARCH 的值设定 NODE_EXPORTER_DOWNLOAD_URL
if [ "$ARCH" = "x86_64" ]; then
    NODE_EXPORTER_DOWNLOAD_URL="$NFS_OPS_BASE_URL/node_exporter-1.8.0.linux-amd64.tar.gz"
elif [ "$ARCH" = "aarch64" ]; then
    NODE_EXPORTER_DOWNLOAD_URL="$NFS_OPS_BASE_URL/node_exporter-1.8.0.linux-arm64.tar.gz"
fi

AUTOCHECKSYS_DOWNLOAD_URL="$NFS_OPS_BASE_URL/autochecksys.tar.gz"
PROMETHEUS_CONFIG_API="$NFS_OPS_BASE_URL/api/integration/prometheus_config/"

VNC_DOWNLOAD_URL1="$NFS_OPS_BASE_URL/init_vnc.sh"
VNC_DOWNLOAD_URL2="$NFS_OPS_BASE_URL/setvncpasswd"
SYS_CONF_DOWNLOAD_URL="$NFS_OPS_BASE_URL/00-nfs-ops.conf"

make_dir() {
    if [ ! -d "$1" ]; then
        echo "目录 $1 不存在。正在创建..."
        mkdir -p "$1"
    else
        echo "目录 $1 已存在。"
    fi
}

install_autochecksys() {
    
    REMOTE_FILE=$AUTOCHECKSYS_DOWNLOAD_URL
    TMP_DIR=/tmp
    TARGET_FILE_NAME="autochecksys.tar.gz"

    curl -o "$TMP_DIR/$TARGET_FILE_NAME" "$REMOTE_FILE"
    tar -xzf "$TMP_DIR/$TARGET_FILE_NAME" -C "$TMP_DIR"
    bash "$TMP_DIR/autochecksys/install.sh" $HOSTNAME $NFS_OPS_BASE_URL || true
    rm "$TMP_DIR/$TARGET_FILE_NAME"
    # rm -rf "$TMP_DIR/autochecksys"

}

install_node_exporter_service() {
    if systemctl is-active node_exporter; then
        echo "node_exporter 服务已启动，无须重复安装。"
        return 0
    fi

    if systemctl is-active firewalld; then
        # 防火墙放行 9100 端口
        firewall-cmd --zone=public --add-port=9100/tcp --permanent
        firewall-cmd --reload
    fi

    REMOTE_FILE=$NODE_EXPORTER_DOWNLOAD_URL
    INSTALL_DIR=/opt/node_exporter
    TMP_FILE="/tmp/node_exporter.tar.gz"
    TMP_DIR="/tmp"

    curl -o "$TMP_FILE" "$REMOTE_FILE"

    tar -xvf $TMP_FILE -C $TMP_DIR

    make_dir $INSTALL_DIR
    cp $(find $TMP_DIR -name "node_exporter" -type f) $INSTALL_DIR
    chmod +x $INSTALL_DIR/node_exporter

    rm -f $TMP_FILE

    do_install_node_exporter_service
}

# 创建并安装 node_exporter 服务的函数
do_install_node_exporter_service() {
    # 定义服务文件路径
    SERVICE_FILE="/etc/systemd/system/node_exporter.service"
    # 定义 node_exporter 的执行路径
    EXEC_PATH="/opt/node_exporter/node_exporter"

    # 检查 node_exporter 可执行文件是否存在
    if [ ! -f "$EXEC_PATH" ]; then
        echo "错误：node_exporter 可执行文件不存在于 $EXEC_PATH。"
        return 1
    fi

    # 创建 node_exporter 用户和组
    id -u node_exporter &>/dev/null || useradd -rs /bin/false node_exporter

    # 创建服务文件
    cat <<EOF > "$SERVICE_FILE"
[Unit]
Description=Node Exporter
Wants=network-online.target
After=network-online.target

[Service]
User=node_exporter
Group=node_exporter
Type=simple
ExecStart=$EXEC_PATH

[Install]
WantedBy=multi-user.target
EOF

    # 重新加载 systemd 配置
    systemctl daemon-reload
    # 启用 node_exporter 服务
    systemctl enable node_exporter.service
    echo "node_exporter 服务已创建并设置为开机启动。"

    # 启动服务
    systemctl start node_exporter.service
}

# 启动 node_exporter 服务的函数
start_node_export_service() {
    # 启动服务
    systemctl start node_exporter.service
}

# 调用 api 配置 prometheus 
call_api() {
    # 发送 curl 请求
    response=$(curl -s -o - --request POST \
        --url "$PROMETHEUS_CONFIG_API" \
        --header 'content-type: application/json' \
        --data '{
            "target": "'${HOSTNAME}:9100'"
        }')

    # 检查 success 字段
    # 提取'success'键的值
    success_value=$(echo "$response" | grep -o '"success":.[^,]*,' | awk -F: '{print $2}' | tr -d ' ",')
    if [ "$success_value" != "true" ]; then
        echo "prometheus 配置失败，返回数据：$response"
        exit 1
    else
        echo "prometheus 配置成功。"
    fi
}
#初始化vnc服务器
init_vnc_server(){
    REMOTE_FILE1=$VNC_DOWNLOAD_URL1
    REMOTE_FILE2=$VNC_DOWNLOAD_URL2
    REMOTE_FILE3=$SYS_CONF_DOWNLOAD_URL
    TMP_DIR=/tmp
    TARGET_FILE_NAME1="init_vnc.sh"
    TARGET_FILE_NAME2="setvncpasswd"
    TARGET_FILE_NAME3="00-nfs-ops.conf"
    curl -o "$TMP_DIR/$TARGET_FILE_NAME1" "$REMOTE_FILE1"
    curl -o "$TMP_DIR/$TARGET_FILE_NAME2" "$REMOTE_FILE2"
    curl -o "/etc/rsyslog.d/$TARGET_FILE_NAME3" "$REMOTE_FILE3"
    bash "$TMP_DIR/init_vnc.sh" $NFS_OPS_BASE_URL
    rm "$TMP_DIR/init_vnc.sh" "$TMP_DIR/setvncpasswd"

}




# 安装 autochecksys
install_autochecksys
# 调用函数来创建并安装 node_exporter 服务
install_node_exporter_service
# 调用运维管理平台的接口设置 prometheus 监控
call_api
init_vnc_server
